#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;

#endregion

//This namespace holds Bars types in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.BarsTypes
{
	public class PointOriginal : BarsType
	{
		public override void ApplyDefaultBasePeriodValue(BarsPeriod period)
		{
		}
		
		public override string ChartLabel(DateTime dateTime)
		{
			return dateTime.ToString("T", Core.Globals.GeneralOptions.CurrentCulture);
		}

		public override void ApplyDefaultValue(BarsPeriod period)
		{
			period.BarsPeriodTypeName = "Tick value";
			period.Value = 2;
		}
					
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description					= @"Range/Renko bars hybrid - Point-O bars hold the range from open to close at a constant level.";
				Name						= "Point-O";
				BarsPeriod					= new BarsPeriod { BarsPeriodType = (BarsPeriodType) 15, BarsPeriodTypeName = "PointOriginal(15)", Value = 2 };
				BuiltFrom					= BarsPeriodType.Tick;
				DaysToLoad					= 3;
				IsIntraday					= true;
			}
			else if (State == State.Configure)
			{
				Name						= string.Format(Core.Globals.GeneralOptions.CurrentCulture, "Point-O {0} ticks", BarsPeriod.Value);

				Properties.Remove(Properties.Find("BaseBarsPeriodType",			true));
				Properties.Remove(Properties.Find("BaseBarsPeriodValue",		true));
				Properties.Remove(Properties.Find("PointAndFigurePriceType",	true));
				Properties.Remove(Properties.Find("ReversalType",				true));
				Properties.Remove(Properties.Find("Value2",						true));

				SetPropertyName("Value", "New Bar (Ticks)" );
			}
		}

		public override int GetInitialLookBackDays(BarsPeriod barsPeriod, TradingHours tradingHours, int barsBack)
		{
			return 1; 
		}

		protected override void OnDataPoint(Bars bars, double open, double high, double low, double close, DateTime time, long volume, bool isBar, double bid, double ask)
		{
			if( SessionIterator == null )  {
				SessionIterator = new SessionIterator(bars);
			}
			
			bool isNewSession = SessionIterator.IsNewSession(time, isBar);
			if (isNewSession) {
				SessionIterator.CalculateTradingDay(time, isBar);
			}
			
			if (bars.Count == 0 || bars.IsResetOnNewTradingDay && isNewSession)
			{
                AddBar( bars, open, high, low, close, time, volume );
			}
			else
			{
              	double tickSize 	= bars.Instrument.MasterInstrument.TickSize;
                double rangeValue 	= Math.Floor(10000000.0 * (double)bars.BarsPeriod.Value * tickSize) / 10000000.0;

				double		barOpen		= bars.GetOpen( bars.Count-1 );
				double		barHigh		= bars.GetHigh( bars.Count-1 );
				double		barLow		= bars.GetLow( bars.Count-1 );
				long		barVolume	= bars.GetVolume( bars.Count-1 );
				
               	double updHigh = Math.Max( barHigh, close );
                double updLow = Math.Min( barLow, close );
				
                if( bars.Instrument.MasterInstrument.Compare( close, barOpen + rangeValue) >= 0 )
				{
					bool	isFirstNewBar	= true;
                    double 	newBarOpen = barOpen + rangeValue;
                    double 	newClose = Math.Min( close, newBarOpen + rangeValue );
                    
					updHigh = newBarOpen;                                      
                    UpdateBar( bars, updHigh, updLow, newClose, time, 0 );

                    do
                    {
                      	AddBar( bars, newBarOpen, newClose, newBarOpen, newClose, time, isFirstNewBar ? volume : 0 );
                      	if( bars.Instrument.MasterInstrument.Compare(close, newClose) == 0 ) {
						   	break; 
					   	}
                       	
						newBarOpen = newClose;
                       	newClose = Math.Min(close,newBarOpen + rangeValue);
                       	isFirstNewBar = false;
                    } 
					while( true );
				}				
               	else if( bars.Instrument.MasterInstrument.Compare( barOpen - rangeValue, close ) >= 0 )
                {
                    bool 	isFirstNewBar = true;
                    double 	newBarOpen = barOpen - rangeValue;
                    double 	newClose = Math.Max( close, newBarOpen - rangeValue);
					
                    updLow = newBarOpen;
                    UpdateBar( bars, updHigh, updLow, newClose, time, 0 );

                    do
                    {
                    	AddBar( bars, newBarOpen, newBarOpen, newClose, newClose, time, isFirstNewBar ? volume : 0 );
                      	if( bars.Instrument.MasterInstrument.Compare( close, newClose ) == 0) {
							break; 
						}

						newBarOpen = newClose;
                       	newClose = Math.Max(close,newBarOpen - rangeValue);
                       	isFirstNewBar = false;
                    }
                    while( true );
				}
				else
				{
                   UpdateBar( bars, updHigh, updLow, close, time, volume );
				}
			}
		}

		public override double GetPercentComplete(Bars bars, DateTime now)
		{
           throw new ApplicationException("GetPercentComplete not supported in " + DisplayName);
		}

	}
}
